@@ -1,5 +1,5 @@
-# WordNet::Similarity::lch.pm version 0.04
-# (Updated 03/25/2003 -- Sid)
+# WordNet::Similarity::lch.pm version 0.05
+# (Updated 06/03/2003 -- Sid)
 #
 # Semantic Similarity Measure package implementing the semantic 
 # relatedness measure described by Leacock and Chodorow (1998).
@@ -44,7 +44,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 
 # 'new' method for the lch class... creates and returns a WordNet::Similarity::lch object.
@@ -81,12 +81,9 @@ sub new
 
     # [trace]
     $self->{'traceString'} = "";
-    if($self->{'trace'})
-    {
-	$self->{'traceString'} .= "WordNet::Similarity::lch object created:\n";
-	$self->{'traceString'} .= "trace :: ".($self->{'trace'})."\n";
-	$self->{'traceString'} .= "cache :: ".($self->{'doCache'})."\n";
-    }
+    $self->{'traceString'} .= "WordNet::Similarity::lch object created:\n";
+    $self->{'traceString'} .= "trace :: ".($self->{'trace'})."\n" if(defined $self->{'trace'});
+    $self->{'traceString'} .= "cache :: ".($self->{'doCache'})."\n" if(defined $self->{'doCache'});
     # [/trace]
 
     return $self;
@@ -121,10 +118,11 @@ sub _initialize
     $self->{'doCache'} = 1;
     $self->{'simCache'} = ();
     $self->{'traceCache'} = ();
+    $self->{'cacheQ'} = ();
+    $self->{'maxCacheSize'} = 1000;
     
     # Initialize tracing.
     $self->{'trace'} = 0;
-    $self->{'traceString'} = "";
 
     # Parse the config file and
     # read parameters from the file.
@@ -280,7 +278,7 @@ sub getRelatedness
     {
 	if(defined $self->{'traceCache'}->{"${wps1}::$wps2"})
 	{
-	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"};
+	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"} if($self->{'trace'});
 	}
 	return $self->{'simCache'}->{"${wps1}::$wps2"};
     }
@@ -288,7 +286,7 @@ sub getRelatedness
     # Now get down to really finding the relatedness of these two.
     $lOffset = $wn->offset($wps1);
     $rOffset = $wn->offset($wps2);
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
 
     if(!$lOffset || !$rOffset)
     {
@@ -376,8 +374,21 @@ sub getRelatedness
     elsif($minDist > 0)
     {
 	$score = -log($minDist/32);
-	$self->{'simCache'}->{"${wps1}::$wps2"} = $score if($self->{'doCache'});
-	$self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'doCache'});
+	if($self->{'doCache'})
+	{
+	    $self->{'simCache'}->{"${wps1}::$wps2"} = $score;
+	    $self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'trace'});
+	    push(@{$self->{'cacheQ'}}, "${wps1}::$wps2");
+	    if($self->{'maxCacheSize'} >= 0)
+	    {
+		while(scalar(@{$self->{'cacheQ'}}) > $self->{'maxCacheSize'})
+		{
+		    my $delItem = shift(@{$self->{'cacheQ'}});
+		    delete $self->{'simCache'}->{$delItem};
+		    delete $self->{'traceCache'}->{$delItem};
+		}
+	    }
+	}
 	return $score;
     }
     else
@@ -395,7 +406,8 @@ sub getTraceString
 {
     my $self = shift;
     my $returnString = $self->{'traceString'};
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
+    $returnString =~ s/\n+$/\n/;
     return $returnString;
 }
 
@@ -528,7 +540,7 @@ sub _printSet
 	$opstr .= "$wps ";
     }
     $opstr =~ s/\s+$//;
-    $self->{'traceString'} .= $opstr;
+    $self->{'traceString'} .= $opstr if($self->{'trace'});
 }
 
 1;
@@ -541,21 +553,21 @@ of word senses using the method described by Leacock and Chodorow (1998).
 
 =head1 SYNOPSIS
 
-use WordNet::Similarity::lch;
+  use WordNet::Similarity::lch;
 
-use WordNet::QueryData;
+  use WordNet::QueryData;
 
-my $wn = WordNet::QueryData->new();
+  my $wn = WordNet::QueryData->new();
 
-my $myobj = WordNet::Similarity::lch->new($wn);
+  my $myobj = WordNet::Similarity::lch->new($wn);
 
-my $value = $myobj->getRelatedness("car#n#1", "bus#n#2");
+  my $value = $myobj->getRelatedness("car#n#1", "bus#n#2");
 
-($error, $errorString) = $myobj->getError();
+  ($error, $errorString) = $myobj->getError();
 
-die "$errorString\n" if($error);
+  die "$errorString\n" if($error);
 
-print "car (sense 1) <-> bus (sense 2) = $value\n";
+  print "car (sense 1) <-> bus (sense 2) = $value\n";
 
 =head1 DESCRIPTION
 
@@ -568,7 +580,7 @@ of this scaled value.
 
 =head1 USAGE
 
-  The semantic relatedness modules in this distribution are built as classes
+The semantic relatedness modules in this distribution are built as classes
 that expose the following methods:
   new()
   getRelatedness()
@@ -579,7 +591,7 @@ See the WordNet::Similarity(3) documentation for details of these methods.
 
 =head1 TYPICAL USAGE EXAMPLES
 
-  To create an object of the lch measure, we would have the following
+To create an object of the lch measure, we would have the following
 lines of code in the perl program. 
 
    use WordNet::Similarity::lch;
@@ -611,13 +623,13 @@ traces are turned off.
 
 =head1 CONFIGURATION FILE
 
-  The behaviour of the measures of semantic relatedness can be controlled by
+The behaviour of the measures of semantic relatedness can be controlled by
 using configuration files. These configuration files specify how certain
 parameters are initialized within the object. A configuration file may be
 specififed as a parameter during the creation of an object using the new
 method. The configuration files must follow a fixed format.
 
-  Every configuration file starts the name of the module ON THE FIRST LINE of
+Every configuration file starts the name of the module ON THE FIRST LINE of
 the file. For example, a configuration file for the lch module will have
 on the first line 'WordNet::Similarity::lch'. This is followed by the various
 parameters, each on a new line and having the form 'name::value'. The
@@ -626,18 +638,20 @@ parameters, each on a new line and having the form 'name::value'. The
 supported in the configuration file. Anything following a '#' is ignored till 
 the end of the line.
 
-  The module parses the configuration file and recognizes the following 
+The module parses the configuration file and recognizes the following 
 parameters:
-  (a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
-      in which case it sets the trace level to 1. Trace level 0 implies
-      no traces. Trace level 1 and 2 imply tracing is 'on', the only 
-      difference being the way in which the synsets are displayed in the 
-      traces. For trace level 1, the synsets are represented as word#pos#sense
-      strings, while for level 2, the synsets are represented as 
-      word#pos#offset strings.
-  (b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' caching. A value of 
-      0 switches caching 'off'. By default caching is enabled.
+  
+(a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
+in which case it sets the trace level to 1. Trace level 0 implies
+no traces. Trace level 1 and 2 imply tracing is 'on', the only 
+difference being the way in which the synsets are displayed in the 
+traces. For trace level 1, the synsets are represented as word#pos#sense
+strings, while for level 2, the synsets are represented as 
+word#pos#offset strings.
+  
+(b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' caching. A value of 
+0 switches caching 'off'. By default caching is enabled.
 
 =head1 SEE ALSO
 
@@ -645,9 +659,11 @@ perl(1), WordNet::Similarity(3), WordNet::QueryData(3)
 
 http://www.d.umn.edu/~patw0006
 
-http://www.cogsci.princeton.edu/~wn/
+http://www.cogsci.princeton.edu/~wn
 
-http://www.ai.mit.edu/people/jrennie/WordNet/
+http://www.ai.mit.edu/people/jrennie/WordNet
+
+http://groups.yahoo.com/group/wn-similarity
 
 =head1 AUTHORS
 
