@@ -1,5 +1,5 @@
-# WordNet::Similarity::lesk.pm version 0.04
-# (Updated 03/25/2003 -- Sid)
+# WordNet::Similarity::lesk.pm version 0.05
+# (Updated 06/03/2003 -- Sid)
 #
 # Module to accept two WordNet synsets and to return a floating point
 # number that indicates how similar those two synsets are, using an
@@ -54,7 +54,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 
 # 'new' method for the lesk class... creates and returns a WordNet::Similarity::lesk object.
@@ -126,10 +126,11 @@ sub _initialize
     $self->{'doCache'} = 1;
     $self->{'simCache'} = ();
     $self->{'traceCache'} = ();
+    $self->{'cacheQ'} = ();
+    $self->{'maxCacheSize'} = 1000;
     
     # Initialize tracing.
     $self->{'trace'} = 0;
-    $self->{'traceString'} = "";
 
     # Stemming? Normalizing?
     $self->{'doStem'} = 0;
@@ -445,16 +446,13 @@ sub _initialize
 
     # [trace]
     $self->{'traceString'} = "";
-    if($self->{'trace'})
-    {
-	$self->{'traceString'} .= "WordNet::Similarity::lesk object created:\n";
-	$self->{'traceString'} .= "trace          :: ".($self->{'trace'})."\n";
-	$self->{'traceString'} .= "cache          :: ".($self->{'doCache'})."\n";
-	$self->{'traceString'} .= "stem           :: ".($self->{'doStem'})."\n";
-	$self->{'traceString'} .= "normalize      :: ".($self->{'doNormalize'})."\n";
-	$self->{'traceString'} .= "relation File  :: $relationFile\n" if($relationFile);
-	$self->{'traceString'} .= "stop File      :: $stopFile\n" if($stopFile);
-    }
+    $self->{'traceString'} .= "WordNet::Similarity::lesk object created:\n";
+    $self->{'traceString'} .= "trace          :: ".($self->{'trace'})."\n" if(defined $self->{'trace'});
+    $self->{'traceString'} .= "cache          :: ".($self->{'doCache'})."\n" if(defined $self->{'doCache'});
+    $self->{'traceString'} .= "stem           :: ".($self->{'doStem'})."\n" if(defined $self->{'doStem'});
+    $self->{'traceString'} .= "normalize      :: ".($self->{'doNormalize'})."\n" if(defined $self->{'doNormalize'});
+    $self->{'traceString'} .= "relation File  :: $relationFile\n" if($relationFile);
+    $self->{'traceString'} .= "stop File      :: $stopFile\n" if($stopFile);
     # [/trace]
 }
 
@@ -513,7 +511,7 @@ sub getRelatedness
     # fact in the cache... if so return the cached value.
     if($self->{'doCache'} && defined $self->{'simCache'}->{"${wps1}::$wps2"})
     {
-	$self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"};
+	$self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"} if($self->{'trace'});
 	return $self->{'simCache'}->{"${wps1}::$wps2"};
     }
 
@@ -707,7 +705,17 @@ sub getRelatedness
     if($self->{'doCache'})
     {
 	$self->{'simCache'}->{"${wps1}::$wps2"} = $score;
-	$self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'};
+	$self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'trace'});
+	push(@{$self->{'cacheQ'}}, "${wps1}::$wps2");
+	if($self->{'maxCacheSize'} >= 0)
+	{
+	    while(scalar(@{$self->{'cacheQ'}}) > $self->{'maxCacheSize'})
+	    {
+		my $delItem = shift(@{$self->{'cacheQ'}});
+		delete $self->{'simCache'}->{$delItem};
+		delete $self->{'traceCache'}->{$delItem};
+	    }
+	}
     }
     
     return $score;
@@ -719,7 +727,8 @@ sub getTraceString
 {
     my $self = shift;
     my $returnString = $self->{'traceString'}."\n";
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
+    $returnString =~ s/\n+$/\n/;
     return $returnString;
 }
 
@@ -748,21 +757,21 @@ of word senses using gloss overlaps as decribed by Banerjee and Pedersen
 
 =head1 SYNOPSIS
 
-use WordNet::Similarity::lesk;
+  use WordNet::Similarity::lesk;
 
-use WordNet::QueryData;
+  use WordNet::QueryData;
 
-my $wn = WordNet::QueryData->new();
+  my $wn = WordNet::QueryData->new();
 
-my $lesk = WordNet::Similarity::lesk->new($wn);
+  my $lesk = WordNet::Similarity::lesk->new($wn);
 
-my $value = $lesk->getRelatedness("car#n#1", "bus#n#2");
+  my $value = $lesk->getRelatedness("car#n#1", "bus#n#2");
 
-($error, $errorString) = $lesk->getError();
+  ($error, $errorString) = $lesk->getError();
 
-die "$errorString\n" if($error);
+  die "$errorString\n" if($error);
 
-print "car (sense 1) <-> bus (sense 2) = $value\n";
+  print "car (sense 1) <-> bus (sense 2) = $value\n";
 
 =head1 DESCRIPTION
 
@@ -775,7 +784,7 @@ lesk measure has been implemented in this module.
 
 =head1 USAGE
 
-  The semantic relatedness modules in this distribution are built as classes
+The semantic relatedness modules in this distribution are built as classes
 that expose the following methods:
   new()
   getRelatedness()
@@ -786,7 +795,7 @@ See the WordNet::Similarity(3) documentation for details of these methods.
 
 =head1 TYPICAL USAGE EXAMPLES
 
-  To create an object of the lesk measure, we would have the following
+To create an object of the lesk measure, we would have the following
 lines of code in the perl program. 
 
    use WordNet::Similarity::lesk;
@@ -818,13 +827,13 @@ traces are turned off.
 
 =head1 CONFIGURATION FILE
 
-  The behaviour of the measures of semantic relatedness can be controlled by
+The behaviour of the measures of semantic relatedness can be controlled by
 using configuration files. These configuration files specify how certain
 parameters are initialized within the object. A configuration file may be
 specififed as a parameter during the creation of an object using the new
 method. The configuration files must follow a fixed format.
 
-  Every configuration file starts the name of the module ON THE FIRST LINE of
+Every configuration file starts the name of the module ON THE FIRST LINE of
 the file. For example, a configuration file for the lesk module will have
 on the first line 'WordNet::Similarity::lesk'. This is followed by the various
 parameters, each on a new line and having the form 'name::value'. The
@@ -833,28 +842,33 @@ parameters, each on a new line and having the form 'name::value'. The
 supported in the configuration file. Anything following a '#' is ignored till
 the end of the line.
 
-  The module parses the configuration file and recognizes the following 
+The module parses the configuration file and recognizes the following 
 parameters:
-  (a) 'trace::' -- The value of this parameter specifies the level of
-      tracing that should be employed for generating the traces. This value
-      is an integer 0, 1 or 2. A value of 0 switches tracing off. A value of
-      1 displays as traces only the gloss overlaps found. A value of 2 displays
-      as traces, all the text being compared.
-  (b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' caching. A value of 
-      0 switches caching 'off'. By default caching is enabled.
-  (c) 'stop::' -- The value is a string that specifies the path of a file 
-      containing a list of stop words that should be ignored for the gloss
-      overlaps.
-  (d) 'stem::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' stemming. A value of 
-      0 switches stemming 'off'. When stemming is enabled, all the words of the
-      glosses are stemmed before their overlaps are determined.
-  (e) 'normalize::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' normalizing of the score. 
-      A value of 0 switches normalizing 'off'. When normalizing is enabled, the 
-      score obtained by counting the gloss overlaps is normalized by the size
-      of the glosses. The details are described in Banerjee Pedersen (2002).
+  
+(a) 'trace::' -- The value of this parameter specifies the level of
+tracing that should be employed for generating the traces. This value
+is an integer 0, 1 or 2. A value of 0 switches tracing off. A value of
+1 displays as traces only the gloss overlaps found. A value of 2 displays
+as traces, all the text being compared.
+  
+(b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' caching. A value of 
+0 switches caching 'off'. By default caching is enabled.
+  
+(c) 'stop::' -- The value is a string that specifies the path of a file 
+containing a list of stop words that should be ignored for the gloss
+overlaps.
+  
+(d) 'stem::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' stemming. A value of 
+0 switches stemming 'off'. When stemming is enabled, all the words of the
+glosses are stemmed before their overlaps are determined.
+  
+(e) 'normalize::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' normalizing of the score. 
+A value of 0 switches normalizing 'off'. When normalizing is enabled, the 
+score obtained by counting the gloss overlaps is normalized by the size
+of the glosses. The details are described in Banerjee Pedersen (2002).
 
 =head1 SEE ALSO
 
@@ -862,9 +876,11 @@ perl(1), WordNet::Similarity(3), WordNet::QueryData(3)
 
 http://www.d.umn.edu/~patw0006
 
-http://www.cogsci.princeton.edu/~wn/
+http://www.cogsci.princeton.edu/~wn
 
-http://www.ai.mit.edu/people/jrennie/WordNet/
+http://www.ai.mit.edu/people/jrennie/WordNet
+
+http://groups.yahoo.com/group/wn-similarity
 
 =head1 AUTHORS
 
