@@ -1,5 +1,5 @@
-# WordNet::Similarity::lin.pm version 0.04
-# (Updated 04/26/2003 -- Sid)
+# WordNet::Similarity::lin.pm version 0.05
+# (Updated 06/03/2003 -- Sid)
 #
 # Semantic Similarity Measure package implementing the semantic 
 # relatedness  measure described by Lin (1998).
@@ -44,7 +44,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 
 # 'new' method for the lin class... creates and returns a WordNet::Similarity::lin object.
@@ -111,10 +111,11 @@ sub _initialize
     $self->{'doCache'} = 1;
     $self->{'simCache'} = ();
     $self->{'traceCache'} = ();
+    $self->{'cacheQ'} = ();
+    $self->{'maxCacheSize'} = 1000;
     
     # Initialize tracing.
     $self->{'trace'} = 0;
-    $self->{'traceString'} = "";
 
     # Parse the config file and
     # read parameters from the file.
@@ -233,13 +234,10 @@ sub _initialize
 
     # [trace]
     $self->{'traceString'} = "";
-    if($self->{'trace'})
-    {
-	$self->{'traceString'} .= "WordNet::Similarity::lin object created:\n";
-	$self->{'traceString'} .= "trace :: ".($self->{'trace'})."\n";
-	$self->{'traceString'} .= "cache :: ".($self->{'doCache'})."\n";
-	$self->{'traceString'} .= "information content file :: $infoContentFile\n";
-    }
+    $self->{'traceString'} .= "WordNet::Similarity::lin object created:\n";
+    $self->{'traceString'} .= "trace :: ".($self->{'trace'})."\n" if(defined $self->{'trace'});
+    $self->{'traceString'} .= "cache :: ".($self->{'doCache'})."\n" if(defined $self->{'doCache'});
+    $self->{'traceString'} .= "information content file :: $infoContentFile\n" if(defined $infoContentFile);
     # [/trace]
 
     # Check for a strange Root_Node_Frequency=0 condition. Normally, not possible.
@@ -446,7 +444,7 @@ sub getRelatedness
     {
 	if(defined $self->{'traceCache'}->{"${wps1}::$wps2"})
 	{
-	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"};
+	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"} if($self->{'trace'});
 	}
 	return $self->{'simCache'}->{"${wps1}::$wps2"};
     }
@@ -454,7 +452,7 @@ sub getRelatedness
     # Now get down to really finding the relatedness of these two.
     $offset1 = $wn->offset($wps1);
     $offset2 = $wn->offset($wps2);
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
 
     if(!$offset1 || !$offset2)
     {
@@ -499,7 +497,26 @@ sub getRelatedness
 		$self->{'traceString'} .= ")  ";
 	    }
 	}
-	$self->{'traceString'} .= "\n\n";
+	$self->{'traceString'} .= "\nConcept1: $wps1 (Freq=";
+	if($self->{'offsetFreq'}->{$pos}->{$offset1})
+	{
+	    $self->{'traceString'} .= $self->{'offsetFreq'}->{$pos}->{$offset1};
+	}
+	else
+	{
+	    $self->{'traceString'} .= "0";
+	}
+	$self->{'traceString'} .= ")\n";
+	$self->{'traceString'} .= "Concept2: $wps2 (Freq=";
+	if($self->{'offsetFreq'}->{$pos}->{$offset2})
+	{
+	    $self->{'traceString'} .= $self->{'offsetFreq'}->{$pos}->{$offset2};
+	}
+	else
+	{
+	    $self->{'traceString'} .= "0";
+	}
+	$self->{'traceString'} .= ")\n\n";
     }
     # [/trace]
 
@@ -521,8 +538,21 @@ sub getRelatedness
     $score = ($minDist ne "-0") ? $minDist : 0;
     $score = ($score == -1) ? 0 : $score;
 
-    $self->{'simCache'}->{"${wps1}::$wps2"} = $score if($self->{'doCache'});
-    $self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'doCache'});
+    if($self->{'doCache'})
+    {
+	$self->{'simCache'}->{"${wps1}::$wps2"} = $score;
+	$self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'trace'});
+	push(@{$self->{'cacheQ'}}, "${wps1}::$wps2");
+	if($self->{'maxCacheSize'} >= 0)
+	{
+	    while(scalar(@{$self->{'cacheQ'}}) > $self->{'maxCacheSize'})
+	    {
+		my $delItem = shift(@{$self->{'cacheQ'}});
+		delete $self->{'simCache'}->{$delItem};
+		delete $self->{'traceCache'}->{$delItem};
+	    }
+	}
+    }
 
     return $score;
 }
@@ -533,7 +563,8 @@ sub getTraceString
 {
     my $self = shift;
     my $returnString = $self->{'traceString'};
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
+    $returnString =~ s/\n+$/\n/;
     return $returnString;
 }
 
@@ -786,7 +817,7 @@ sub _printSet
 	$opstr .= "$wps ";
     }
     $opstr =~ s/\s+$//;
-    $self->{'traceString'} .= $opstr;
+    $self->{'traceString'} .= $opstr if($self->{'trace'});
 }
 
 1;
@@ -800,21 +831,21 @@ Lin (1998).
 
 =head1 SYNOPSIS
 
-use WordNet::Similarity::lin;
+  use WordNet::Similarity::lin;
 
-use WordNet::QueryData;
+  use WordNet::QueryData;
 
-my $wn = WordNet::QueryData->new();
+  my $wn = WordNet::QueryData->new();
 
-my $mymeasure = WordNet::Similarity::lin->new($wn);
+  my $mymeasure = WordNet::Similarity::lin->new($wn);
 
-my $value = $mymeasure->getRelatedness("car#n#1", "bus#n#2");
+  my $value = $mymeasure->getRelatedness("car#n#1", "bus#n#2");
 
-($error, $errorString) = $mymeasure->getError();
+  ($error, $errorString) = $mymeasure->getError();
 
-die "$errorString\n" if($error);
+  die "$errorString\n" if($error);
 
-print "car (sense 1) <-> bus (sense 2) = $value\n";
+  print "car (sense 1) <-> bus (sense 2) = $value\n";
 
 =head1 DESCRIPTION
 
@@ -825,7 +856,7 @@ this measure of semantic relatedness of concepts.
 
 =head1 USAGE
 
-  The semantic relatedness modules in this distribution are built as classes
+The semantic relatedness modules in this distribution are built as classes
 that expose the following methods:
   new()
   getRelatedness()
@@ -836,7 +867,7 @@ See the WordNet::Similarity(3) documentation for details of these methods.
 
 =head1 TYPICAL USAGE EXAMPLES
 
-  To create an object of the lin measure, we would have the following
+To create an object of the lin measure, we would have the following
 lines of code in the perl program. 
 
    use WordNet::Similarity::lin;
@@ -868,13 +899,13 @@ traces are turned off.
 
 =head1 CONFIGURATION FILE
 
-  The behaviour of the measures of semantic relatedness can be controlled by
+The behaviour of the measures of semantic relatedness can be controlled by
 using configuration files. These configuration files specify how certain
 parameters are initialized within the object. A configuration file may be
 specififed as a parameter during the creation of an object using the new
 method. The configuration files must follow a fixed format.
 
-  Every configuration file starts the name of the module ON THE FIRST LINE of
+Every configuration file starts the name of the module ON THE FIRST LINE of
 the file. For example, a configuration file for the lin module will have
 on the first line 'WordNet::Similarity::lin'. This is followed by the various
 parameters, each on a new line and having the form 'name::value'. The
@@ -883,22 +914,25 @@ parameters, each on a new line and having the form 'name::value'. The
 supported in the configuration file. Anything following a '#' is ignored till
 the end of the line.
 
-  The module parses the configuration file and recognizes the following 
+The module parses the configuration file and recognizes the following 
 parameters:
-  (a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
-      in which case it sets the trace level to 1. Trace level 0 implies
-      no traces. Trace level 1 and 2 imply tracing is 'on', the only 
-      difference being the way in which the synsets are displayed in the 
-      traces. For trace level 1, the synsets are represented as word#pos#sense
-      strings, while for level 2, the synsets are represented as 
-      word#pos#offset strings.
-  (b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' caching. A value of 
-      0 switches caching 'off'. By default caching is enabled.
-  (c) 'infocontent::' -- The value for this parameter should be a string that
-      specifies the path of an information content file containing the 
-      frequency of occurrence of every WordNet concept in a large corpus. The
-      format of this file is specified in a later section.
+
+(a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
+in which case it sets the trace level to 1. Trace level 0 implies
+no traces. Trace level 1 and 2 imply tracing is 'on', the only 
+difference being the way in which the synsets are displayed in the 
+traces. For trace level 1, the synsets are represented as word#pos#sense
+strings, while for level 2, the synsets are represented as 
+word#pos#offset strings.
+  
+(b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' caching. A value of 
+0 switches caching 'off'. By default caching is enabled.
+  
+(c) 'infocontent::' -- The value for this parameter should be a string that
+specifies the path of an information content file containing the 
+frequency of occurrence of every WordNet concept in a large corpus. The
+format of this file is specified in a later section.
 
 =head1 INFORMATION CONTENT
 
@@ -921,30 +955,30 @@ of the format of these files follows. The FIRST LINE of this file MUST contain
 the version of WordNet that the file was created with. This should be present 
 as a string of the form 
 
-wnver::<version>
+  wnver::<version>
 
 For example, if WordNet version 1.7.1 was used for creation of the
 information content file, the following line would be present at the start
 of the information content file.
 
-wnver::1.7.1
+  wnver::1.7.1
 
 The rest of the file contains on each line a WordNet synset offset, 
 part-of-speech and a frequency count, in the form
 
-<offset><part-of-speech> <frequency> [ROOT]
+  <offset><part-of-speech> <frequency> [ROOT]
 
 without any leading or trailing spaces. For example, one of the lines of an
 information content file may be as follows.
 
-63723n 667
+  63723n 667
 
 where '63723' is a noun synset offset and 667 is its frequency
 count. Suppose the noun synset with offset 1740 is the root node of one of 
 the noun taxonomies and has a frequency count of 17625. Then this synset would 
 appear in an information content file as follows:
 
-1740n 17625 ROOT
+  1740n 17625 ROOT
 
 The ROOT tags are extremely significant in determining the top of the 
 hierarchies and must not be omitted. Typically, frequency counts for the noun
@@ -956,9 +990,11 @@ perl(1), WordNet::Similarity(3), WordNet::QueryData(3)
 
 http://www.d.umn.edu/~patw0006
 
-http://www.cogsci.princeton.edu/~wn/
+http://www.cogsci.princeton.edu/~wn
 
-http://www.ai.mit.edu/people/jrennie/WordNet/
+http://www.ai.mit.edu/people/jrennie/WordNet
+
+http://groups.yahoo.com/group/wn-similarity
 
 =head1 AUTHORS
 
