@@ -1,5 +1,5 @@
-# WordNet::Similarity::random.pm version 0.04
-# (Updated 03/25/2003 -- Sid)
+# WordNet::Similarity::random.pm version 0.05
+# (Updated 06/03/2003 -- Sid)
 #
 # Random semantic distance generator module.
 #
@@ -43,7 +43,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 
 # 'new' method for the random class... creates and returns a WordNet::Similarity::random object.
@@ -115,7 +115,6 @@ sub _initialize
     
     # Initialize tracing.
     $self->{'trace'} = 0;
-    $self->{'traceString'} = "";
 
     # Parse the config file and
     # read parameters from the file.
@@ -182,13 +181,10 @@ sub _initialize
 
     # [trace]
     $self->{'traceString'} = "";
-    if($self->{'trace'})
-    {
-	$self->{'traceString'} .= "WordNet::Similarity::lch object created:\n";
-	$self->{'traceString'} .= "trace   :: ".($self->{'trace'})."\n";
-	$self->{'traceString'} .= "cache   :: ".($self->{'doCache'})."\n";
-	$self->{'traceString'} .= "maxRand :: ".($self->{'maxrand'})."\n";
-    }
+    $self->{'traceString'} .= "WordNet::Similarity::lch object created:\n";
+    $self->{'traceString'} .= "trace   :: ".($self->{'trace'})."\n" if(defined $self->{'trace'});
+    $self->{'traceString'} .= "cache   :: ".($self->{'doCache'})."\n" if(defined $self->{'doCache'});
+    $self->{'traceString'} .= "maxRand :: ".($self->{'maxrand'})."\n" if(defined $self->{'maxrand'});
     # [/trace]
 }
 
@@ -260,7 +256,7 @@ sub getRelatedness
     {
 	if(defined $self->{'traceCache'}->{"${wps1}::$wps2"})
 	{
-	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"};
+	    $self->{'traceString'} = $self->{'traceCache'}->{"${wps1}::$wps2"} if($self->{'trace'});
 	}
 	return $self->{'simCache'}->{"${wps1}::$wps2"};
     }
@@ -268,7 +264,7 @@ sub getRelatedness
     # Now get down to really finding the relatedness of these two.
     $offset1 = $wn->offset($wps1);
     $offset2 = $wn->offset($wps2);
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
 
     if(!$offset1 || !$offset2)
     {
@@ -278,9 +274,9 @@ sub getRelatedness
 	return undef;
     }
 
-    $score = sprintf("%.3f", (($self->{'maxrand'})?(rand($self->{'maxrand'})):(rand)));
+    $score = sprintf("%.3f", ((defined $self->{'maxrand'})?(rand($self->{'maxrand'})):(rand)));
     $self->{'simCache'}->{"${wps1}::$wps2"} = $score if($self->{'doCache'});
-    $self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'doCache'});
+    $self->{'traceCache'}->{"${wps1}::$wps2"} = $self->{'traceString'} if($self->{'doCache'} &&  $self->{'trace'});
 
     return $score;
 }
@@ -291,7 +287,8 @@ sub getTraceString
 {
     my $self = shift;
     my $returnString = $self->{'traceString'};
-    $self->{'traceString'} = "";
+    $self->{'traceString'} = "" if($self->{'trace'});
+    $returnString =~ s/\n+$/\n/;
     return $returnString;
 }
 
@@ -319,21 +316,21 @@ of word senses using a random measure.
 
 =head1 SYNOPSIS
 
-use WordNet::Similarity::random;
+  use WordNet::Similarity::random;
 
-use WordNet::QueryData;
+  use WordNet::QueryData;
 
-my $wn = WordNet::QueryData->new();
+  my $wn = WordNet::QueryData->new();
 
-my $random = WordNet::Similarity::random->new($wn);
+  my $random = WordNet::Similarity::random->new($wn);
 
-my $value = $random->getRelatedness("car#n#1", "bus#n#2");
+  my $value = $random->getRelatedness("car#n#1", "bus#n#2");
 
-($error, $errorString) = $random->getError();
+  ($error, $errorString) = $random->getError();
 
-die "$errorString\n" if($error);
+  die "$errorString\n" if($error);
 
-print "car (sense 1) <-> bus (sense 2) = $value\n";
+  print "car (sense 1) <-> bus (sense 2) = $value\n";
 
 =head1 DESCRIPTION
 
@@ -345,7 +342,7 @@ word sense pair every time.
 
 =head1 USAGE
 
-  The semantic relatedness modules in this distribution are built as classes
+The semantic relatedness modules in this distribution are built as classes
 that expose the following methods:
   new()
   getRelatedness()
@@ -356,7 +353,7 @@ See the WordNet::Similarity(3) documentation for details of these methods.
 
 =head1 TYPICAL USAGE EXAMPLES
 
-  To create an object of the random measure, we would have the following
+To create an object of the random measure, we would have the following
 lines of code in the perl program. 
 
    use WordNet::Similarity::random;
@@ -388,13 +385,13 @@ traces are turned off.
 
 =head1 CONFIGURATION FILE
 
-  The behaviour of the measures of semantic relatedness can be controlled by
+The behaviour of the measures of semantic relatedness can be controlled by
 using configuration files. These configuration files specify how certain
 parameters are initialized within the object. A configuration file may be
 specififed as a parameter during the creation of an object using the new
 method. The configuration files must follow a fixed format.
 
-  Every configuration file starts the name of the module ON THE FIRST LINE of
+Every configuration file starts the name of the module ON THE FIRST LINE of
 the file. For example, a configuration file for the random module will have
 on the first line 'WordNet::Similarity::random'. This is followed by the various
 parameters, each on a new line and having the form 'name::value'. The
@@ -403,22 +400,25 @@ parameters, each on a new line and having the form 'name::value'. The
 supported in the configuration file. Anything following a '#' is ignored till
 the end of the line.
 
-  The module parses the configuration file and recognizes the following 
+The module parses the configuration file and recognizes the following 
 parameters:
-  (a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
-      in which case it sets the trace level to 1. Trace level 0 implies
-      no traces. Trace level 1 and 2 imply tracing is 'on' the only 
-      difference being in the way in which the synsets are displayed in the 
-      output. For trace level 1, the synsets are represented as word#pos#sense
-      strings, while for level 2, the synsets are represented as 
-      word#pos#offset strings.
-  (b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
-      which case it takes the value 1, i.e. switches 'on' caching. A value of 
-      0 switches caching 'off'. By default caching is enabled. For module 
-      enabling caching implies that a previously generated random value will
-      be reused if the same word sense pair occurs again. If caching is disabled
-      for every word sense pair a new random number will be generated.
-  (c) 'maxrand::' -- specifies the maximum random number that can be generated.
+  
+(a) 'trace::' -- can take values 0, 1 or 2 or the value can be omitted,
+in which case it sets the trace level to 1. Trace level 0 implies
+no traces. Trace level 1 and 2 imply tracing is 'on' the only 
+difference being in the way in which the synsets are displayed in the 
+output. For trace level 1, the synsets are represented as word#pos#sense
+strings, while for level 2, the synsets are represented as 
+word#pos#offset strings.
+  
+(b) 'cache::' -- can take values 0 or 1 or the value can be omitted, in 
+which case it takes the value 1, i.e. switches 'on' caching. A value of 
+0 switches caching 'off'. By default caching is enabled. For module 
+enabling caching implies that a previously generated random value will
+be reused if the same word sense pair occurs again. If caching is disabled
+for every word sense pair a new random number will be generated.
+  
+(c) 'maxrand::' -- specifies the maximum random number that can be generated.
 
 =head1 SEE ALSO
 
@@ -426,9 +426,11 @@ perl(1), WordNet::Similarity(3), WordNet::QueryData(3)
 
 http://www.d.umn.edu/~patw0006
 
-http://www.cogsci.princeton.edu/~wn/
+http://www.cogsci.princeton.edu/~wn
 
-http://www.ai.mit.edu/people/jrennie/WordNet/
+http://www.ai.mit.edu/people/jrennie/WordNet
+
+http://groups.yahoo.com/group/wn-similarity
 
 =head1 AUTHORS
 
